function[CCC_value,k]=conditional_CCC(data,var_cause,var_effect,INFO_CCC)
%Estimates conditional Compression Complexity Causality(CCC) measure from
%input time series variable 'var_cause' to 'var_effect' in the 'data' matrix 
%Outputs-
%   1. CCC_value - Estimated time averaged causality using Compression Complexity
%   Causality(CCC) from variable var_cause to var_effect
%   2. k - no. of times the difference between two dynamical complexities
%   is taken for computing averaged CCC
%
%Inputs - 
%  1. data - input matrix of real numbers or a symbolic sequence of
%          integers, contains multivariate data in the form
%          varables*time points
%  2. var_cause - driver variable
%  3. var_effect - target variable
%  4. INFO_CCC is a structure taking in input parameters - 
%  N=length of input series (number of time points), L=Past window size, w=current window size, delta=Jump step size for each window, 
%  B (no. of bins) = is a row vector/scalar to specify Number of Bins to partition the input data matrix to create a symbolic sequence,
%             if this is set to 0 then input is already a symbolic sequence, if it is a scalar then each variable is binned using that 
%             value, if it is a row vector and the ith entry entry is 'k', then 'k' specifies the
%             no. of bins for the ith variable
%Calls functions ETC and Partition. Check the readme file for more details.
%
%
%Example - A trial of the experiment when coupled AR processes are generated, X is the dependent variable and Y is independent, N = 1000,
%INFO_CCC = struct('L',150,'w',15,'delta',80,'B',2,'N',N); the values are
%set in the Main.m function
%
% >>  [CCC_value,k]=CCC([X; Y],2,1,INFO_CCC) gives CCC_value = 0.0627 and k = 12      (As Y causes X, the CCC_value>0)
% >>  [CCC_value,k]=CCC([X; Y],1,2,INFO_CCC) gives CCC_value = -0.0028 and k = 12      (As X does not cause Y, the CCC_value~0 and << CC_value from X to Y)
%
% Aditi Kathpalia, Nithin Nagaraj, NIAS, October 2017 (v1.0)
% August 11, 2018 - Aditi@NIAS, Incuded the feature for conditional CCC (v2.0)

LEN_past=INFO_CCC.L;
ADD_meas=INFO_CCC.w;
Num_bins=INFO_CCC.B;
LEN=INFO_CCC.N;
STEP_size=INFO_CCC.delta;

LEN_to_check=LEN_past+ADD_meas;
n_vars=size(data,1);


% creating multivariate matrix without the driver (causal) variable,
% (var_cause)
if var_cause==1
    data_reduced=data(2:end,:);
elseif var_cause==n_vars
        data_reduced=data(1:end-1,:);
    else
    data_reduced=[data(1:var_cause-1,:); data(var_cause+1:end,:)];
end
        
%sequence1=X;
%sequence2=Y;

    k=1;
    for i=1:STEP_size:LEN-LEN_to_check
        
        ETC_ini(k)=ETC(data_reduced(:,i:i+LEN_past-1),Num_bins);
        ETC_ini(k)=ETC_ini(k)/(LEN_past-1);
        ETC_ini_2D(k)=ETC(data(:,i:i+LEN_past-1),Num_bins);
        ETC_ini_2D(k)=ETC_ini_2D(k)/(LEN_past-1);
        ETC_fin(k)=ETC([data_reduced(:,i:i+LEN_past-1) repmat(data(var_effect,(i+LEN_past:i+LEN_to_check-1)),n_vars-1,1)],Num_bins);
        ETC_fin(k)=ETC_fin(k)/(LEN_to_check-1);
        ETC_fin_2D(k)=ETC([data(:,i:i+LEN_past-1) repmat(data(var_effect,(i+LEN_past:i+LEN_to_check-1)),n_vars,1)],Num_bins);
        ETC_fin_2D(k)=ETC_fin_2D(k)/(LEN_to_check-1);
        
        delta_ETC(k)=ETC_fin(k)-ETC_ini(k);
        delta_ETC_2D(k)=ETC_fin_2D(k)-ETC_ini_2D(k);

        k=k+1;
    end
%     figure;plot(delta_ETC,'b');
%     hold on;plot(delta_ETC_2D,'r');
%     To estimate average difference between complexity rates and hence CCC
        
    sum_delta_ETC=sum(delta_ETC);
    sum_delta_ETC_2D=sum(delta_ETC_2D);
    
   CCC_value=(sum_delta_ETC-sum_delta_ETC_2D)/(k-1);
  